/* copyright (C) cameo nettech {{{2
 * Filename:	memmap.c
 *
 * Author:	arrow <jianzhang@nettech-global.com>
 * Created at:  Sat Jul  4 16:50:40 2009
 * }}}*/
/*header files	{{{1*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <linux/mm_types.h>
#include <linux/sched.h>
#include <linux/mmzone.h>
#include <linux/pagemap.h>
#include <linux/seq_file.h>
/*
#include <linux/config.h>
*/
#include <linux/kprobes.h>
#include <linux/smp_lock.h>

#include <asm/traps.h>
#include <asm/ptrace.h>
#include <asm/stacktrace.h>
#include <cameo/define.h>
#include <cameo/color_dump.h>
#include <cameo/env_dump.h>
#include <cameo/memmap_pid.h>
#include <cameo/memrw.h>
/*}}}*/

/*declaration		{{{1*/
#define LEN_FILE_PATH_DUMP	128

int memmap_pid_dump(pid_t pid, struct pt_regs *regs);
static ssize_t memmap_pid_read(struct file *file, char __user *buffer, size_t buflen, loff_t *fpos);
static ssize_t memmap_pid_write(struct file * file, const char __user *buf, size_t count, loff_t *ppos);
static int memmap_pid_open(struct inode *inode, struct file *filp);
static inline void mem_dump_stack(struct task_struct *task, struct pt_regs *regs, struct vm_area_struct *vma);
static void pid_trace_callback(struct task_struct *task, struct pt_regs *regs);

int phy_memmap_pid = 0;
int sta_memmap_pid = 0;
int cnt_memmap_pid = 1;
int do_stack_dump = 1;
/*}}}*/
/* TODO:
 * 1. use seq-file to read msg
 * 2. bind to pid
 * */

/*functions		{{{1*/
struct mem_size_stats
{
	unsigned long resident;
	unsigned long shared_clean;
	unsigned long shared_dirty;
	unsigned long private_clean;
	unsigned long private_dirty;
};

static void smaps_pte_range(struct vm_area_struct *vma, pmd_t *pmd,
				unsigned long addr, unsigned long end,
				struct mem_size_stats *mss)
{
	pte_t *pte, ptent;
	spinlock_t *ptl;
	struct page *page;
	int i = 0, cnt_phy = 0;
	char *wrap_detail[] = {"", "\n"};

	pte = pte_offset_map_lock(vma->vm_mm, pmd, addr, &ptl);
	do {
		if ((i%16) == 0 && (sta_memmap_pid != 0 || phy_memmap_pid != 0)) { /* ? can use bit move */
			seep("\n");
		}
		i ++;
		ptent = *pte;
		if (phy_memmap_pid != 0) {
			seep(" pte: %p, ptent: %08lx : ", pte, pte_val(ptent));
		}
		if (!pte_present(ptent)) {
			if (sta_memmap_pid != 0) {
				seep(" p%s", wrap_detail[!(!phy_memmap_pid)]);
			}
			continue;
		}

		mss->resident += PAGE_SIZE;

		/* be from mm/memery.c, should export this func,
		 * if we are act as module */
		page = vm_normal_page(vma, addr, ptent);
		if (!page) {
			if (sta_memmap_pid != 0) {
				seep(" n%s", wrap_detail[!(!phy_memmap_pid)]);
			}
			continue;
		}

		cnt_phy ++;
		/* in fact: mss not used currently */
		if (page_mapcount(page) >= 2) {
			if (pte_dirty(ptent))
				mss->shared_dirty += PAGE_SIZE;
			else
				mss->shared_clean += PAGE_SIZE;
		} else {
			if (pte_dirty(ptent))
				mss->private_dirty += PAGE_SIZE;
			else
				mss->private_clean += PAGE_SIZE;
		}
		if (sta_memmap_pid != 0) {
			seep(" %d%s", page_mapcount(page), wrap_detail[!(!phy_memmap_pid)]);
		}
	} while (pte++, addr += PAGE_SIZE, addr != end);
	pte_unmap_unlock(pte - 1, ptl);
	if (cnt_memmap_pid != 0) {
		seep("\n page cnt:%4d,%9lu B,%5lu KB", i, PAGE_SIZE * i, (PAGE_SIZE * i)>>10);
		seep("\n phy  cnt:%4d,%9lu B,%5lu KB\n", cnt_phy, PAGE_SIZE * i, (PAGE_SIZE * cnt_phy)>>10);
	}
	cond_resched();
}

static inline void smaps_pmd_range(struct vm_area_struct *vma, pud_t *pud,
				unsigned long addr, unsigned long end,
				struct mem_size_stats *mss)
{
	pmd_t *pmd;
	unsigned long next;

	/* convert to "page middle directory" */
	pmd = pmd_offset(pud, addr);
	do {
		next = pmd_addr_end(addr, end);
		if (pmd_none_or_clear_bad(pmd))
			continue;
		smaps_pte_range(vma, pmd, addr, next, mss);
	} while (pmd++, addr = next, addr != end);
}

static inline void smaps_pud_range(struct vm_area_struct *vma, pgd_t *pgd,
				unsigned long addr, unsigned long end,
				struct mem_size_stats *mss)
{
	pud_t *pud;
	unsigned long next;

	pud = pud_offset(pgd, addr);
	do {
		next = pud_addr_end(addr, end);
		if (pud_none_or_clear_bad(pud))
			continue;
		smaps_pmd_range(vma, pud, addr, next, mss);
	} while (pud++, addr = next, addr != end);
}

static void phy_mem_dump(struct vm_area_struct *vma)
{
	pgd_t *pgd;
	unsigned long next;
	unsigned long addr, end;
	struct mem_size_stats mss;

	addr = vma->vm_start;
	end = vma->vm_end;
	/* seep("\t[ %lu : %lu KB ]", end - addr, (end - addr)>>10); */
	/* get out "page directory */
	pgd = pgd_offset(vma->vm_mm, addr);
	do {
		next = pgd_addr_end(addr, end);
		if (pgd_none_or_clear_bad(pgd))
			continue;
		smaps_pud_range(vma, pgd, addr, next, &mss);
	} while (pgd++, addr = next, addr != end);
}

static void vma_dump(struct vm_area_struct *vma)
{
	int flags;
	struct file *file;
	unsigned long ino;
	dev_t dev;
	char buff[LEN_FILE_PATH_DUMP];
	char *p;
	struct inode *inode;
	struct mm_struct *mm = vma->vm_mm;

	while (vma) {
		flags = vma->vm_flags;
		file = vma->vm_file;
		ino = 0;
		dev = 0;

		if (file) {
			inode = vma->vm_file->f_dentry->d_inode;
			dev = inode->i_sb->s_dev;
			ino = inode->i_ino;
		}

		seepd("# %s%08lx-%08lx%s %c%c%c%c %08lx %02x:%02x %lu\t\t",
				get_color(CL_GREEN),
				vma->vm_start,
				vma->vm_end,
				get_color(CL_END),
				flags & VM_READ ? 'r' : '-',
				flags & VM_WRITE ? 'w' : '-',
				flags & VM_EXEC ? 'x' : '-',
				flags & VM_MAYSHARE ? 's' : 'p',
				vma->vm_pgoff << PAGE_SHIFT,
				MAJOR(dev), MINOR(dev), ino);
		if (file) {
			p = d_path(file->f_dentry, file->f_vfsmnt, buff, sizeof(buff));
			if (!IS_ERR(p)) {
				seepd("%s", p);
			}
		} else {
			const char *name = arch_vma_name(vma);
			if (!name) {
				if (mm) {
					if (vma->vm_start <= mm->start_brk &&
							vma->vm_end >= mm->brk) {
						name = "[heap]";
					} else if (vma->vm_start <= mm->start_stack &&
						   vma->vm_end >= mm->start_stack) {
						name = "[stack]";
					}
				} else {
					name = "[vdso]";
				}
			}
			if (name) {
				seepd("%s", name);
			}
		}
		phy_mem_dump(vma);
		seepd("\n");
		vma = vma->vm_next;
	}
}

static void pid_msg_dump(struct task_struct *task, struct pt_regs *regs)
{
	struct mm_struct *mm;
	struct vm_area_struct *vma/*, *tail_vma*/;

	seep("comm: %s%s%s\n", get_color(CL_GREEN), task->comm, get_color(CL_END));
	seep("tgid: %d\n", task->tgid);
	mm = get_task_mm(task);
	if (!mm)
		return;

	/* tail_vma = get_gate_vma(task); */
	/* down_read(&mm->mmap_sem); */
	/* TODO: lock mm, while reading */
	vma = mm->mmap;

	/* struct mm_struct *mm = vma->vm_mm; */
	vma_dump(vma);

	if ((regs != NULL) && (do_stack_dump != 0)) {
		mem_dump_stack(task, task_pt_regs(task), vma);
		/* show_registers(task_pt_regs(task)); */ /* this will
							     backtrace, do not use it */
		show_regs(task_pt_regs(task));
		/*CAMEOTAG:Add by Juncheng for cdd handling crash*/
        cdd_crash_regs(task_pt_regs(task));
	}
	seep("\n\nnormal finish stack dump\n");
	pid_trace_callback(task, regs);
	/*CAMEOTAG:Add by Juncheng for cdd handling crash*/
    cdd_crash_trace_callback(task, regs);
}

static void other_pid_mem_hex_dump(struct task_struct *task, unsigned char *buf, size_t len)
{
	size_t i;
	unsigned long tmp;
	int copied;

	/* ?? align buf(addr) ?*/
	for (i = 0; i < len; i++) {
		if (i && !(i % 4))
			seepd("\n");
		copied = access_process_vm(task, buf, &tmp, sizeof(tmp), 0);
		if (copied != sizeof(tmp)) {
			break;
		}
		seepd("%08x ", tmp);
		buf += sizeof(tmp);
	}
	seepd("\n");
}
static inline void mem_dump_stack(struct task_struct *task,
	struct pt_regs *regs, struct vm_area_struct *vma)
{
	unsigned long len;
	unsigned long start_sp;
#define LEN_DUMP_MAX_STACK ((unsigned long)(1*1024))

	start_sp = vma->vm_mm?vma->vm_mm->start_stack:0;
	/*
	seep("stack_start: 0x%x\n", vma->vm_mm?vma->vm_mm->start_stack:0);
	seep("sp: 0x%x\n", (unsigned char*)regs->regs[29]);
	*/

	if (start_sp == 0) {
		return;
	}
	len = vma->vm_mm->start_stack - regs->regs[29];
	len = min(len, LEN_DUMP_MAX_STACK);
	other_pid_mem_hex_dump(task, (unsigned char*)regs->regs[29], len);
}

int memmap_pid_dump(pid_t pid, struct pt_regs *regs)
{
	int error = -ESRCH;
	struct task_struct *task;

	seepd("will dump memmap_pid: %d\n", pid);
	lock_kernel();	/* lock kernel to hold died task's resource */
	task = find_task_by_pid(pid);
	if (task == NULL) {
		seep("pid not exist: %d\n", pid);
		goto fail_find_pid;
	}
	get_task_struct(task);	/* hold the task */
	pid_msg_dump(task, regs);
	put_task_struct(task);

fail_find_pid:
	unlock_kernel();
	return error;
}
EXPORT_SYMBOL(memmap_pid_dump);

static ssize_t memmap_pid_read(struct file *file, char __user *buffer, size_t buflen, loff_t *fpos)
{
	seepd("write pid to it\n");
	return 0;
}

static ssize_t memmap_pid_write(struct file * file, const char __user *buf, size_t count, loff_t *ppos)
{
	pid_t pid, pid_buf, *p;

	p = &pid_buf;
	if (copy_from_user(p, buf, sizeof(pid_t))) {
		goto fail_copy;
	}
	pid = simple_strtoul((char *)p, (char **)&p, 0);
	if (pid > 0) {
		memmap_pid_dump(pid, NULL);
	} else {
		seepd("pid invalidate: %d\n", pid);
	}
fail_copy:
	return count;
}

static int memmap_pid_open(struct inode *inode, struct file *filp)
{
	return 0;
}

unsigned long get_valsp(unsigned long epc, int *ret_offset, unsigned long *ret_val, unsigned long sp,
	struct task_struct *task, unsigned long cmp)
{
	unsigned long ret = 0;
	unsigned long orig_epc = epc;

	int copied;
	unsigned long tmp;
	int offset;

	//*ret_offset = *ret_val = 0;
/*        copied = access_process_vm(task, buf, &tmp, sizeof(tmp), 0);*/

	if (epc&0x3) {
		seepd("bad epc, should algin with 4Bytes\n");
		goto fail;
	}
	for (;epc;) {
		if (epc <= 0x400000) {
			seep("ecp too low, abort: %08lx\n", epc);
			break;
		}
		/* read exec code */
		copied = access_process_vm(task, epc, &tmp, sizeof(tmp), 0);
		if (copied != sizeof(tmp)) {
			break;
		}
		epc -= sizeof(epc);	/* move size of one ins */
		if ((tmp&0xffff0000) == cmp) { /* yes */
			*ret_offset = offset = (short)(tmp&0x0000ffff);
			/* read stack data */
			if (offset < 0) {	/* skip, this maybe read SP size */
				ret = epc;
				break;
			}
			copied = access_process_vm(task, sp + offset, &tmp, sizeof(tmp), 0);
			if (copied != sizeof(tmp)) {
				break;
			}
			*ret_val = tmp;
			ret = epc;	/* ok */
			break;
		}
		if ((tmp&0x3c1c0000) == 0x3c1c0000) {
			/* suppose at the top of the function */
			ret = orig_epc;	/* if correct, this should be the first, else the orignal
					   logic is wrong
					   the RA keep the as input */
			*ret_offset = 0;
			break;
		}
		if (orig_epc - epc >= 0x100000) {
			seepd("epc move too large, abort %08lx -> %08lx\n", orig_epc, epc);
			break;
		}
	}

fail:
	return ret;
}

static void pid_trace_callback(struct task_struct *task, struct pt_regs *regs)
{
	struct pt_regs *reg_thread = task_pt_regs(task);
	unsigned long epc_val = 0, epc_saved, ra_val, dummy;
	int ret = 0, ra_pos = 0, sp_size = 0;
	unsigned long sp = reg_thread->regs[29];

	epc_val = reg_thread->cp0_epc;
	dummy = ra_val = reg_thread->regs[31];	/* RA init for leave function */

	do {
		/* first, get RA, then sp_size */
		/*   also get RA-Position when get RA,
		 *        get stack size at second read */
		epc_saved = epc_val;
		ret = get_valsp(epc_val, &ra_pos, &ra_val, sp, task, 0xafbf0000);//, (unsigned char*)regs->regs[29]
		if (ret <= 0) {
			break;
		}
		epc_val = ret;
		ret = get_valsp(epc_val, &sp_size, &dummy, sp, task, 0x27bd0000);
		if (ret <= 0) {
			break;
		}
		if (sp_size < 0) {	/* abs */
			sp_size = -sp_size;
		}
		seep("%08lx (size: %ld, sp: %08lx, ra: %08lx)\n", epc_saved, sp_size, sp, ra_val);/* TODO: show argv0-3*/
		epc_val = ra_val;	/* when leave function, first call get_valsp, will keep use init RA */
		sp += sp_size;
		if (epc_saved == epc_val) {
			break;
		}
	} while (ret > 0);
	seep("all backtrace dump fine, only FYI\n");
}

const struct file_operations proc_memmap_pid_operations = {
	.read		= memmap_pid_read,
	.open		= memmap_pid_open,
	.write		= memmap_pid_write,
};

/* vim:fdm=marker:ts=8:ft=c:norl:fdl=1:
 * }}}*/

